/**
 * 为hy2.0项目定制的构建方案，包括：
 * - 从js入口文件检索scss/css依赖并去重，生成打包后的css
 * - 拆分js为lib.js和index.js，提升构建速度
 * - 为以上构建流程编写的开发服务
 * @author jiao.shen
 */
'use strict';
var extend = require('extend'),
    Qunar = require('./ykit-config-qunar'),
    path = require('path'),
    fs = require('fs'),
    normalize = require('normalize-path'),
    HappyPack = require('happypack'),
    babelQuery = require('./consts').babelQuery,
    packServiceInitiator = require('./ykit_config/pack');

exports.version = require('./package.json').version;

exports.config = function (options, cwd) {
    Qunar.config.call(this, options, cwd);
    cwd = normalize(cwd);
    var self = this;
    var dir = cwd.split('/');
    var projectDir = dir[dir.length - 1];

    // 增加cache配置
    babelQuery.cacheDirectory = true;
    // happypack
    var happyPackPlugin = new HappyPack({
        loaders: [
            {
                loader: require.resolve('babel-loader'),
                test: /\.js$/,
                exclude: /node_modules/,
                query: babelQuery
            }
        ],
        threads: 4,
        verbose: false,
        cacheContext: {
            env: process.env.NODE_ENV
        },
        tempDir: path.join(__dirname, '../.happypack'),
        cachePath: path.join(__dirname, '../.happypack/cache--[id].json')
    });
    // [配置]编译环境
    var env = 'dev';
    switch (this.env) {
    case 'local':
        env = 'dev';
        break;
    case 'dev':
        env = 'production';
        break;
    case 'prd':
        env = 'production';
    }
    var envPlugin = new this.webpack.DefinePlugin({
        "process.env": {
            NODE_ENV: JSON.stringify(env)
        }
    });
    // dll配置以及初始化命令
    if (fs.existsSync(path.resolve(cwd, '.lib_cache', 'manifest.json'))) {
        var dllPlugin = new this.webpack.DllReferencePlugin({
            context: cwd,
            manifest: JSON.parse(fs.readFileSync(path.resolve(cwd, '.lib_cache', 'manifest.json'), 'utf8'))
        });
        this.config.plugins.push(dllPlugin);
    }
    this.config.plugins.push(envPlugin);
    this.config.plugins.push(happyPackPlugin);
    // source-map
    this.config.devtool = 'source-map';
    // js loaders
    this.config.module.loaders.push({
        test: /\.js$/,
        exclude: /(node_modules|bower_components)/,
        loader: 'happypack/loader'
    });
    // 由于babel-loader的缓存导致经常会出现引用错误，这里用一个空loader防止异常
    this.config.module.loaders.push({
        test: /\.(scss|css)$/,
        loader: require.resolve('./loader/empty-loader')
    });
    // css dev编译
    // lib.js静态服务
    this.config.module.preLoaders.push({
        test: /\.js$/, loaders: [
            require.resolve('./loader/component-shortcut'),
            require.resolve('./loader/scss-preloader'),
            require.resolve('./loader/async-require')
        ]
    });
    // 连接本地提示
    if (this.env === 'local') {
        this.config.module.postLoaders.push({
            test: /\.(js)$/,
            loader: sysPath.join(__dirname, './loader/notice-postloader.js')
        });
    }

    var packService = packServiceInitiator(cwd, self, dllPlugin);
    this.applyMiddleware(packService.devMiddleware);
    this.applyBeforePack(packService.beforePack);
    this.packCallbacks.push(packService.cssPack);
    this.packCallbacks.push(packService.libPack);
    this.packCallbacks.push(function() {
        var swPath = sysPath.join(cwd, 'npm-shrinkwrap.json');
        var yarnPath = sysPath.join(cwd, 'yarn.lock');
        if (!fs.existsSync(yarnPath) && !fs.existsSync(swPath)) {
            console.log('\n[Ykit] yarn.lock 或者 npm-shrinkwrap.json 不存在，构建存在风险，建议使用 yarn 进行模块安装或执行 ykit sw 生成 npm-shrinkwrap.json')
            console.log('[Ykit] 文档请参考：http://ued.qunar.com/ykit/plugins-qunar.html\n')
        }
    });

    this.commands.push({
        name: 'setup',
        module: require('./commands/setup.js')
    });

    this.commands.push({
        name: 'dll',
        module: {
            usage: '生成lib.js开发版本代码',
            run: function () {
                packService.getDll(null, false, self.config.getVendor);
                console.log('ykit-config-yo: dev版本lib.js构建成功，请重新启动ykit server.');
            }
        }
    });

    this.commands.push({
        name: 'dll-min',
        module: {
            usage: '生成lib.js production环境代码',
            run: function () {
                packService.getDll(null, true, self.config.getVendor);
                console.log('ykit-config-yo: production版本lib.js构建成功，请重新启动ykit server。');
            }
        }
    });

    this.commands.push({
        name: 'shrinkwrap',
        abbr: 'sw',
        module: require('./commands/shrinkwrap.js')
    });
};
